package
{
	import com.georg.IterateGUI;
	import com.georg.Mandelbrot;
	import com.georg.Stepper;
	import com.georg.tools.FPSCounter;
	import flash.events.KeyboardEvent;
	
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	/**
	 * ...
	 * @author Georg Graf
	 */
	
	[SWF(width="600", height="400", backgroundColor="#000000", frameRate="30")]
	public class Main extends Sprite
	{
		public static var stage:Stage;
		
		private var mb:Mandelbrot;
		
		private var scale:Number;
		private var colorFactor:Number;
		
		private var stepper:Stepper;
		private var iterateGUI:IterateGUI;
		
		private var fpscounter:FPSCounter;
		
		// dragging vars
		private var dragTimer:Timer;
		private var dragging:Boolean;
		private var dragPrevX:int;
		private var dragPrevY:int;
		
		public function Main() 
		{
			// init stage
			Main.stage = stage;
			
			// create Mandelbrot
			mb = new Mandelbrot(600, 400);
			addChild(mb);
			
			// add interaction
			mb.addEventListener(MouseEvent.MOUSE_DOWN, mbDownHandler);
			mb.addEventListener(MouseEvent.MOUSE_UP, mbUpHandler);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, stageWheelListener);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, toggleIterateGUI);
			// create Stepper
			stepper = new Stepper(20);
			stepper.addEventListener(Event.CHANGE, draw);
			stepper.x = 5;
			stepper.y = 5;
			addChild(stepper);
			
			// create fps counter
			fpscounter = new FPSCounter();
			fpscounter.x = stage.stageWidth - 90;
			fpscounter.y = 10;
			addChild(fpscounter);
			
			// create iterate gui
			iterateGUI = new IterateGUI(mb);
			addChild(iterateGUI);
			
			// create one picture
			scale = .01;
			colorFactor = 0;
			stageClick(false);
			
			// create drag timer
			dragging = false;
			dragTimer = new Timer(200, 1);
			dragTimer.addEventListener(TimerEvent.TIMER_COMPLETE, setDrag);
		}
		
		protected function toggleIterateGUI(e:KeyboardEvent):void 
		{
			if (e.keyCode == 73) // i
				iterateGUI.visible = !iterateGUI.visible;
		}
		private function stageWheelListener(e:MouseEvent):void 
		{
			
			scale -= scale * (.05 * e.delta);
			
			stageClick();
		}
		
		private function setDrag(e:TimerEvent):void 
		{
			dragPrevX = stage.mouseX;
			dragPrevY = stage.mouseY;
			dragging = true;
			stage.addEventListener(MouseEvent.MOUSE_MOVE, drag);
		}
		
		private function mbUpHandler(e:MouseEvent = null):void 
		{
			dragTimer.stop();
			
			if (dragging) // DRAGGING END
			{
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, drag);
				dragging = false;
				stageClick();
			}
			else // CLICK
			{
				stageClick();
			}
		}
		private function mbDownHandler(e:MouseEvent = null):void 
		{
			dragTimer.reset();
			dragTimer.start();
		}
		
		private function drag(e:MouseEvent):void 
		{
			mb.drag(stage.mouseX - dragPrevX, stage.mouseY - dragPrevY);
			
			dragPrevX = stage.mouseX;
			dragPrevY = stage.mouseY;
		}
		
		private function stageClick(setOffset:Boolean = true):void
		{
			if (iterateGUI.blob.hitTestPoint(stage.mouseX, stage.mouseY))
				return;
			

			if (setOffset)
			{
				//mb.offsetX = stage.mouseX - (stage.stageWidth / 2);
				//mb.offsetY = stage.mouseY - (stage.stageHeight / 2);
			}
			draw();
			
			//scale -= scale * .05;
			colorFactor += .05;
			if (colorFactor > 1) colorFactor = 0;
		}
		
		private function draw(e:Event = null):void
		{
			// start frame counter
			fpscounter.start();
			
			//mb.drawIncremental(10, 0, stepper.value, scale, (colorFactor * 360), drawComplete);
			mb.drawPixelIncremental(100, 0, stepper.value, scale, (colorFactor * 360), drawComplete);
			iterateGUI.drawIterations();
		}
		
		private function drawComplete():void
		{
			// stop frame counter
			fpscounter.stop();
		}
		
	}
}