package com.georg
{
	import com.georg.tools.ComplexNumber;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	public class IterateGUI extends Sprite
	{
		public static var DISPLAY_RAD:Number = 100;
		
		protected var mb:Mandelbrot;
		public var blob:Sprite;
		
		public function IterateGUI(mb:Mandelbrot)
		{
			this.mb = mb;
			this.mouseEnabled = false;
			
			addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		protected function init(e:Event):void
		{
			createBlob();
			
			blob.addEventListener(MouseEvent.MOUSE_DOWN, mDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, mUp);
		}
		
		protected function createBlob():void
		{
			blob = new Sprite();
			blob.buttonMode = true;
			blob.x = stage.stageWidth / 2;
			blob.y = stage.stageWidth / 2;
			
			blob.graphics.beginFill(0x00DDDD);
			blob.graphics.lineStyle(2, 0xFFFFFF);
			blob.graphics.drawCircle(0, 0, 5);
			blob.graphics.endFill();
			
			addChild(blob);
		}
		
		protected function mDown(e:MouseEvent):void
		{
			stage.addEventListener(MouseEvent.MOUSE_MOVE, displayIterations);
		}
		
		protected function mUp(e:MouseEvent):void
		{
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, displayIterations);
		}
		
		protected function displayIterations(e:MouseEvent = null):void
		{
			blob.x = stage.mouseX;
			blob.y = stage.mouseY;
		
			drawIterations();
		}
		
		public function drawIterations():void
		{
			var arr:Array = mb.getIterationsForPoint(mb.mouseX - (mb.width / 2), mb.mouseY - (mb.height / 2));
			var cnZ:ComplexNumber;
			
			var pos:Point = new Point(blob.x + ((stage.stageWidth / 2) - blob.x) / 2, blob.y + ((stage.stageHeight / 2) - blob.y) / 2);
 
			// draw iterations (lines)
			graphics.clear();
			graphics.lineStyle(1, 0x00AAAA, 0.5);
			graphics.moveTo(blob.x, blob.y);
			
			for each(cnZ in arr)
			{
				graphics.lineTo(pos.x + (cnZ.im * DISPLAY_RAD), pos.y + (cnZ.re * DISPLAY_RAD));
			}
			graphics.endFill();
			
			// draw iterations (circles)
			graphics.beginFill(0xDDDD00, 0.8);
			graphics.lineStyle();
			
			for each(cnZ in arr)
			{
				graphics.drawCircle(pos.x + (cnZ.im * DISPLAY_RAD), pos.y + (cnZ.re * DISPLAY_RAD), 2);
			}
			graphics.endFill();
		}
	}
}